
/*
 * Copyright (c) 2005 by Pascal de Bruijn 
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General 
 * Public License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using System;
using System.IO;
using System.Diagnostics;
using Gtk;
using Gnome;
using Gnome.Vfs;
using Glade;
using Mono.Unix;

public class Corbicula
{
  [Widget] Gtk.Window CorbiculaWindow;
  [Widget] Gtk.TreeView ScanTargetTreeView;
  [Widget] Gtk.Image TranslateThisApplicationMenuItemImage;
  
  ListStore store;
  
  public Corbicula (string [] args)
  {    
    Program Program = new Program ("Corbicula", "0.2", Modules.UI, args);
    
    Glade.XML GladeXML = new Glade.XML (null, "corbicula.glade", "CorbiculaWindow", "corbicula");   
    GladeXML.Autoconnect (this);    
    CorbiculaWindow.Icon = new Gdk.Pixbuf (null, "corbicula.png"); 

    try
    {
      TranslateThisApplicationMenuItemImage.Pixbuf = new Gdk.Pixbuf ("/usr/share/pixmaps/lpi-translate.png");
    }
    catch (Exception ex)
    {
      // don't display the image
    }
    
    ScanTargetTreeView.HeadersVisible = true;

    TreeViewColumn IconColumn = new TreeViewColumn ();
    CellRenderer IconColumnRenderer = new CellRendererPixbuf ();
    IconColumn.Title = "";
    IconColumn.PackStart (IconColumnRenderer, true);
    IconColumn.AddAttribute (IconColumnRenderer, "pixbuf", 0);
    ScanTargetTreeView.AppendColumn (IconColumn);
    
    TreeViewColumn TargetColumn = new TreeViewColumn ();
    CellRenderer TargetColumnRenderer = new CellRendererText ();
    TargetColumn.Title = Catalog.GetString("Target");
    TargetColumn.PackStart (TargetColumnRenderer, true);
    TargetColumn.AddAttribute (TargetColumnRenderer, "text", 1);
    ScanTargetTreeView.AppendColumn (TargetColumn);

    Config config = new Config ();
    if (config.ShowUnixPath)
    {
      TreeViewColumn MediaColumn = new TreeViewColumn ();
      CellRenderer MediaColumnRenderer = new CellRendererText ();
      MediaColumn.Title = Catalog.GetString("Where");
      MediaColumn.PackStart (MediaColumnRenderer, true);
      MediaColumn.AddAttribute (MediaColumnRenderer, "text", 2);
      ScanTargetTreeView.AppendColumn (MediaColumn);
    }

    PopulateScanTargetStore (ref store);
        
    VolumeMonitor vm = VolumeMonitor.Get ();
    
    vm.DriveConnected += OnDriveConnected;
    vm.DriveDisconnected += OnDriveDisconnected;
    vm.VolumeMounted += OnVolumeMounted;
    vm.VolumeUnmounted += OnVolumeUnmounted;
    
    Program.Run ();
  }
  
  private void PopulateScanTargetStore(ref ListStore store)
  {
    store = new ListStore (typeof (Gdk.Pixbuf), typeof (string), typeof (string));
      
    // FIXME: Hardcoded stuff
    store.AppendValues (new Gdk.Pixbuf("/usr/share/icons/gnome/24x24/filesystems/gnome-fs-desktop.png"), "Desktop", Environment.GetFolderPath (Environment.SpecialFolder.Personal) + "/Desktop");
    store.AppendValues (new Gdk.Pixbuf("/usr/share/icons/gnome/24x24/filesystems/gnome-fs-home.png"), "Home Folder", Environment.GetFolderPath (Environment.SpecialFolder.Personal));

    if (System.IO.Directory.Exists(Environment.GetFolderPath (Environment.SpecialFolder.Personal) + "/Documents"))
    {    
      store.AppendValues (new Gdk.Pixbuf("/usr/share/icons/gnome/24x24/filesystems/gnome-fs-directory.png"), "Documents", Environment.GetFolderPath (Environment.SpecialFolder.Personal) + "/Documents");
    }
    
    Bookmarks b = new Bookmarks ();
    foreach (string s in b.BookmarkList)
    {
      store.AppendValues (new Gdk.Pixbuf("/usr/share/icons/gnome/24x24/filesystems/gnome-fs-directory.png"), new Gnome.Vfs.Uri (s).ExtractShortName (), Gnome.Vfs.Uri.GetLocalPathFromUri (s));
    }
        
    VolumeMonitor vm = VolumeMonitor.Get ();

    foreach (Volume v in vm.MountedVolumes)
    {
      // FIXME: ActivationUri compare should not be hardcoded!
      if (v.IsMounted && Gnome.Vfs.Uri.GetLocalPathFromUri (v.ActivationUri) == "/" || new Gnome.Vfs.Uri (v.ActivationUri).ExtractDirname() == "/media")
      {
        // FIXME: work around for Ubuntu 5.04/5.10
        if (v.DisplayName != "Root Volume (2)")
        {
          store.AppendValues (new Gdk.Pixbuf("/usr/share/icons/gnome/24x24/devices/" + v.Icon + ".png"), v.DisplayName, Gnome.Vfs.Uri.GetLocalPathFromUri (v.ActivationUri));
        }
      }
    }
        
    ScanTargetTreeView.Model = store;    
  }
	
  private void OnCorbiculaWindowDeleteEvent (object o, DeleteEventArgs args)
  {
    Gtk.Application.Quit ();
  }
  
  private void OnDriveConnected (object o, DriveConnectedArgs args)
  {
    PopulateScanTargetStore (ref store);
  }
  
  private void OnDriveDisconnected (object o, DriveDisconnectedArgs args)
  {
    PopulateScanTargetStore (ref store);
  }
  
  private void OnVolumeMounted (object o, VolumeMountedArgs args)
  {
    PopulateScanTargetStore (ref store);
  }
  
  private void OnVolumeUnmounted (object o, VolumeUnmountedArgs args)
  {
    PopulateScanTargetStore (ref store);
  }

  private void OnScanMenuItemActivate (object o, EventArgs args)
  {
    TreeIter iter;
    TreeModel model;
    
    if ((ScanTargetTreeView.Selection).GetSelected (out model, out iter))
    {
      ListStore store = new ListStore (typeof (string), typeof (string), typeof(bool));
        
      new Scanning ((string) model.GetValue (iter, 2), ref store);
    }  
  }
  
  private void OnQuitMenuItemActivate (object o, EventArgs args)
  {
    Gtk.Application.Quit ();
  }
  
  private void OnPreferencesMenuItemActivate (object o, EventArgs args)
  {
    new Preferences ();
  }
  
  private void OnContentsMenuItemActivate (object o, EventArgs args)
  {
    // FIXME: Still need to write documentation
    Process.Start ("yelp", "");
  }
  
  private void OnTranslateThisApplicationMenuItemActivate (object o, EventArgs args)
  {
    Process.Start ("gnome-open", "https://launchpad.net/products/corbicula/+translations");
  }
  
  private void OnAboutMenuItemActivate (object o, EventArgs args)
  {
    new About ();
  }
}