
/*
 * Copyright (c) 2005 by Pascal de Bruijn 
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General 
 * Public License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using System;
using System.IO;
using System.Threading;
using Gtk;
using ClamAv;
using Mono.Unix;

public class ScanEventArgs : EventArgs
{
  public string FileName;
  
  public ScanEventArgs (string FileName)
  {
    this.FileName = FileName;
  }
}

public delegate void ScanEventHandler (object o, ScanEventArgs args);
public delegate void ScanFinishedEventHandler (object o, EventArgs args); 

public class ScanThread
{
  string Path;
  int Depth;
  ListStore store;
 
  public event ScanEventHandler ScanEvent;
  public event ScanFinishedEventHandler ScanFinishedEvent;

  public ScanThread (string Path, int Depth, ref ListStore store)
  {
    this.Path = Path;
    this.Depth = Depth;
    this.store = store;
  }
  
  public void Scan ()
  {
    ScanPath(this.Path, this.Depth, ref this.store);
    
    EventArgs args = new EventArgs ();
    ScanFinishedEvent (this, args);
  }
  
  public void ScanPath (string Path, int Depth, ref ListStore store)
  {  
    Console.WriteLine(Path);

    ClamAv.ClamAv cl = new ClamAv.ClamAv ();
	try
    {
      // FIXME: Path shouldn't be hardcoded!
      if ((Path != "/dev") && (Path != "/.dev") && (Path != "/proc") && (Path != "/sys")) 
      {
       
        string[] Files = Directory.GetFiles (Path);    
        foreach (string File in Files)
        {
          try
          {                   
            UnixFileInfo FileInfo = new UnixFileInfo (File);
            
            if (!FileInfo.IsBlockDevice && !FileInfo.IsCharacterDevice && !FileInfo.IsFIFO && 
                !FileInfo.IsSymbolicLink && !FileInfo.IsSocket)
               
            {

              Console.WriteLine(File);

              ScanEventArgs args = new ScanEventArgs(File);
              ScanEvent (this, args);
              
              string VirusName;
              if (cl.ScanFile (File, out VirusName))
              {
                store.AppendValues (File, VirusName, false);
              } 
            }
          }
          catch (ClamAvException ex)
          {
            Console.WriteLine (File + " Caused: " + ex.Message);
          }
          catch (UnauthorizedAccessException ex)
          {
            Console.WriteLine (File + " Caused: " + ex.Message);
          }
        }

        try
        {
          string[] Dirs = Directory.GetDirectories (Path);
          foreach (string Dir in Dirs)
          {
            Console.WriteLine(Dir + " - " + Depth);
            if (Depth > 0)
            {
              ScanPath(Dir, Depth - 1, ref store);
            }
          }
        } 
        catch (UnauthorizedAccessException ex)
        {
          Console.WriteLine (Path + " Caused: " + ex.Message);
        }
      }
    }
    finally
    {
      cl.Dispose ();
    }        
  }
}