
/*
 * Copyright (c) 2005 by Pascal de Bruijn 
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General 
 * Public License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using System;
using System.IO;
using System.Threading;
using Gtk;
using Gdk;
using Gnome;
using Glade;
using ClamAv;
using Mono.Unix;

public class Scanning
{   
  [Widget] Gtk.Dialog ScanningDialog;
  [Widget] Gtk.Label FileScannedLabel;
  [Widget] Gtk.ProgressBar ProgressBar;
  [Widget] Gtk.Button DeleteSelectedButton;
  
  [Widget] Gtk.Dialog ResultsDialog;
  [Widget] Gtk.TreeView ResultsTreeView;
   
  string FileName;
  
  Thread t;
  
  ListStore store;
  
  public Scanning (string Path, ref ListStore store)
  {
    Glade.XML gaxml = new Glade.XML (null, "corbicula.glade", "ScanningDialog", "corbicula");
    gaxml.Autoconnect (this);
    ScanningDialog.Icon = new Gdk.Pixbuf (null, "corbicula.png");
    
    ResultsTreeView.HeadersVisible = true;

    TreeViewColumn FileColumn = new TreeViewColumn ();
    CellRendererText FileColumnRenderer = new CellRendererText ();
    FileColumn.Title = Catalog.GetString("File");
    FileColumn.PackStart (FileColumnRenderer, true);
    FileColumn.AddAttribute (FileColumnRenderer, "text", 0);
    ResultsTreeView.AppendColumn (FileColumn);

    TreeViewColumn VirusColumn = new TreeViewColumn ();
    CellRendererText VirusColumnRenderer = new CellRendererText ();
    VirusColumn.Title = Catalog.GetString("Infected by");
    VirusColumn.PackStart (VirusColumnRenderer, true);
    VirusColumn.AddAttribute (VirusColumnRenderer, "text", 1);
    ResultsTreeView.AppendColumn (VirusColumn);

    TreeViewColumn DeleteColumn = new TreeViewColumn ();
    CellRendererToggle DeleteColumnRenderer = new CellRendererToggle ();
    DeleteColumnRenderer.Activatable = true;
    DeleteColumnRenderer.Toggled += OnToggleToggled;
    DeleteColumn.Title = Catalog.GetString("Delete");
    DeleteColumn.PackStart (DeleteColumnRenderer, true);
    DeleteColumn.AddAttribute (DeleteColumnRenderer, "active", 2);
    ResultsTreeView.AppendColumn (DeleteColumn);

    ResultsTreeView.Model = store;
    this.store = store;   

    Config config = new Config ();    
    int RecursionDepth = config.RecursionDepth;

    ScanThread st = new ScanThread (Path, RecursionDepth, ref store);
    st.ScanEvent += OnScanEvent;
    st.ScanFinishedEvent += OnScanFinishedEvent;
    (t = new Thread (new ThreadStart (st.Scan))).Start ();
  }
  
  private void OnScanFinishedEvent (object o, EventArgs args)
  {
    Console.WriteLine("Done!");
    DeleteSelectedButton.Sensitive = true;
  }
  
  private void OnScanEvent (object o, ScanEventArgs args)
  {
    this.FileName = args.FileName;
    GLib.Timeout.Add (0, new GLib.TimeoutHandler (UpdateProgressBar));
  }
  
  private bool UpdateProgressBar ()
  {
    ProgressBar.Pulse ();
    FileScannedLabel.Text = this.FileName;
    return false;
  }
     
  private void OnToggleToggled (object o, ToggledArgs args)
  {
    TreeIter iter;

    if (store.GetIter (out iter, new TreePath (args.Path))) 
    {
      store.SetValue (iter, 2, !((bool) store.GetValue (iter, 2)));
    }
  }
  
  private void DeleteInfectedFiles ()
  {
    foreach (object[] row in store) 
    {
      if ((bool) row[2])
      {
        try
        {
          File.Delete ((string) row[0]);
        }
        catch (System.Security.SecurityException ex)
        {
          Console.WriteLine (row[0] + ": delete: Caused SecurityException!");
        }
        catch (UnauthorizedAccessException ex)
        {
          Console.WriteLine (row[0] + ": delete: Caused UnauthorizedAccessException!");
        }
      }  
    }
  }

  private void OnDeleteSelectedButtonClicked (object o, EventArgs args)
  {
    DeleteInfectedFiles ();
    ScanningDialog.Destroy ();
  }

  private void OnCloseButtonClicked (object o, EventArgs args)
  {
    t.Abort ();
    ScanningDialog.Destroy ();
  }
  
  private void OnHelpButtonClicked (object o, EventArgs args)
  {
  }
}